'===========================================================================
' 25-February-2002    VBScript file for Windows Installer
'===========================================================================
'
' Description:
' ============
' This VBS script file contains methods intended to be called from a Windows
' Installer installation package or merge module. It is written to be called
' from a package created by Wise for Windows Installer, but could be adapted
' to be called from a package created with Installshield Windows Installer.
' Typically, methods in this file are called from a "Custom Action" in the
' Windows Installer package.
'
' NOTE: You cannot pass parameters from a Custom Action to VBScript methods.
' You must use the script to read and write properties to the installation;
' the script can read and write the current installation session properties.
' In Wise for Windows Installer, this is performed using Session.Property.
' Properties in the Windows Installer package can be READ as follows:
'   local-variable = Session.Property("MY_INSTALL_PROPERTY")
' Properties in the Windows Installer package can be WRITTEN as follows:
'   Session.Property("MY_INSTALL_PROPERTY") = "new-value"
' Note that property names must be all UPPER-CASE to be visible in scripts.
' Properties that are being set by these functions do not necessarily need
' to already exist in the installation package. The property will be created
' if it does not already exist.
'
' VBScript methods cannot return values when called from a Custom Action, but
' can set a success or failure return code, by setting the function value to
' either ERROR_SUCCESS or ERROR_INSTALL_FAILURE. To cause the installation to
' be rolled back, a custom action that returns ERROR_INSTALL_FAILURE needs to
' be called before the InstallFinalize action. After that action, it will
' still cause the installation to terminate with an error message, but it
' will not cause a rollback (uninstallation).
'
'===========================================================================
' This file contains both Public and Private methods.
'
' The Private methods typically should not be changed for each installation
' package; the Public methods are the methods that are called from an
' installation, and are typically specific to (and need to be edited for)
' an installation package.
' Search for "EXAMPLE USAGE:" in this file for sections to customize.
'
' NOTE: In Merge Modules, Session properties always have a suffix code.
' For example, when used with a merge module (.MSM) installation, a session
' property is referenced similar to:
'   Session.Property("_MTS_PACKAGE.5F109603_3401_2FD4_EC17_77C04F3C2B1A")
' and in a standard (.MSI) installation, a session property is referenced
' similar to:
'   Session.Property("_MTS_PACKAGE")
'===========================================================================
'
'
'===========================================================================
' How to create a custom action to call a VBScript file that is stored in the
' Binary table of your installation. Use the Binary table as the source if you
' want to call the file during installation, but after installation the file
' is not needed. [During installation, the VBScript file is extracted to a
' temporary directory, it is called, and then it is deleted.]
'
' In Wise for Windows Installer:
' To call a VBScript method from your installation:
'   1.  Go to the Actions tab in Setup Editor.
'   2.  Right-click the All Custom Actions folder icon in the left pane, and
'       select New > Custom Action from the right-click menu.
'       The Custom Action Type dialog box appears.
'   3.  In the Custom Action Type drop-down list, select
'           VBScript Call (Requires Runtime).
'   4.  In the Source drop-down list, select Stored in Binary Table.
'   5.  Click the Next button.
'       The Custom Action Target dialog box appears.
'   6.  Fill out the options in the Custom Action Target dialog box:
'       -Custom Action Name. Enter a unique name for the custom action, making
'       sure it does not match the name of any standard Windows Installer action
'       (check the Windows Installer SDK Help). The name you enter can contain
'       letters, numbers, underscores, and periods. It must begin with either
'       a letter or an underscore.
'       -Script File. Specify the path and name of (or browse to) a file to call
'       during installation. It will be copied to the Binary table.
'       -Script function call. Type the name of the function within the VBScript
'       file to call. NOTE: No parameters can be passed to the function, and the
'       function cannot return any values (Session properties must be used for
'       getting and setting values in the installation.)
'   7.  Click the Next button in the Custom Action Target dialog box.
'       The Custom Action Location dialog box appears.
'   8.  In this dialog box, specify the location in the action sequences where
'       you want to place your custom action. For details on the Custom Action
'       Location dialog box, see the help file (Choosing Custom Action Location
'       and Conditions).
'   8.  Click the Next button.
'   9.  Click the Next button.
'       The Custom Action Properties dialog box appears.
'   10. In this dialog box, specify options that determine when and how the
'       custom action is executed.
'   11. Click the Finish button.
'===========================================================================
' WARNING: In a Merge Module, the same script file CANNOT be included more
' than once (for example, for 2 different custom actions). [In a standard
' installation (.MSI), the same script file can be included multiple times.]
' If you attempt to compile a Merge Module that does have the same script
' file included more than once, you may receive an error such as "cannot
' save the file because of insufficient disk space". If the same script file
' is needed more than once in a merge module, make a copy of the file with
' a different name.
'===========================================================================
' The comments can be deleted from "production" versions of the file, if desired.
'===========================================================================
'
'===========================================================================
' If you encounter an "Error 1720" WITHOUT a function name or custom action
' name when running an install that includes this script file, the most likely
' cause is a syntax error in this file (that prevents it from being "compiled"
' at run-time).
'===========================================================================


Option Explicit

' Windows Installer Custom Action return values
Const ERROR_SUCCESS = 0
Const ERROR_INSTALL_FAILURE = 1603
Const ERROR_FUNCTION_FAILED = 1627
' MsgBox icon styles (don't need to be declared, already known in VBScript):
'vbCritical (16) Display Critical Message icon.
'vbQuestion (32) Display Warning Query icon.
'vbExclamation (48) Display Warning Message icon.
'vbInformation (64) Display Information Message icon.

Dim g_MsgBoxTitle
'   Set the value of this global variable by calling a function (in a VBScript
'   file, the variable cannot be assigned a value outside of a function in this
'   declarations section).
Call SET_MSGTITLE

Private Function SET_MSGTITLE()
    g_MsgBoxTitle = "Installing " & Session.Property("ProductName") & _
        " - version:" & Session.Property("ProductVersion")
End Function



'===========================================================================
' PUBLIC FUNCTION (calls private function "set_MTSPackage")
' Purpose:  Delete MTS (Component Services) package.
' Effects:  Deletes MTS (Component Services) package.
'===========================================================================
Function DeleteMTSPackage()
    Dim strName
    DeleteMTSPackage = ERROR_SUCCESS

' EXAMPLE USAGE:
' ##############
' Note: This example assumes it is being called from a merge module
    strName = Session.Property("_MTS_PACKAGE_NAME.5F109603_3401_2FD4_EC17_77C04F3C2B1A")
    Call set_MTSPackage(strName, "False", "", "", "", "", "")

    ' Terminate install on error
    If Err.Number <> 0 Then
        DeleteMTSPackage = ERROR_INSTALL_FAILURE
        Exit Function
    End If
End Function

'===========================================================================
' PUBLIC FUNCTION (calls private function "set_MTSPackage")
' Purpose:  Create an MTS (Component Services) package, and install DLL COM
'           components in the package.
' Effects:  Creates MTS (Component Services) package.
'===========================================================================
Function CreateMTSPackage()
    Dim strName, strPath, arrDLLs, strTimeout, strDescription, strCreator
    CreateMTSPackage = ERROR_SUCCESS

' EXAMPLE USAGE:
' ##############
' Note: This example assumes it is being called from a merge module
    ' For example, if DLL's are installed in Program Files\Common Files\MyFolder
    strPath = Session.Property("MyFolder.5F109603_3401_2FD4_EC17_77C04F3C2B1A")

    strName = Session.Property("_MTS_PACKAGE_NAME.5F109603_3401_2FD4_EC17_77C04F3C2B1A")
    ' The property _MTS_DLL_NAMES is a comma-delimited string of DLL names, for example:
    '        "MyDLL1.DLL,MyDLL2.DLL,MyDLL3.DLL"
    arrDLLs = Split(Session.Property("_MTS_DLL_NAMES.5F109603_3401_2FD4_EC17_77C04F3C2B1A"), ",")
    ' The following could also be read from session properties, instead of being
    ' hard-coded in this script file.
    strTimeout = "90"
    strDescription = "My Company Package"
    strCreator = "My Company, Inc."

    Call set_MTSPackage(strName, "True", strPath, arrDLLs, strTimeout, strDescription, strCreator)

    ' Terminate install on error
    If Err.Number <> 0 Then
        CreateMTSPackage = ERROR_INSTALL_FAILURE
        Exit Function
    End If
End Function



'===========================================================================
' Purpose:  Create MTS package, set properties, and install DLL's in package.
' Inputs:   strName: Name of MTS package
'           blnAdd: False to delete package, True to add new package (Note:
'                   package will be deleted if it exists before being added)
'           strPath: File path for physical location of components (Note:
'                   all components are required to be in the same directory)
'           arrDLLs: Array of DLL names
'           strTimeout: MTS Transaction timeout value (in seconds)
'           strDescription: Description of MTS package
'           strCreator: String for "Created by" text in MTS package
' Returns:  None.
' Errors:   Error raised if there are problems communicating with MTS or IIS.
' NOTE:     Activation of MTS package is set to "Local" (Server package), and
'           identity is set to IIS anonymous username and password.
'===========================================================================
Private Function set_MTSPackage(ByVal strName, ByVal blnAdd, ByVal strPath, _
        arrDLLs, ByVal strTimeout, ByVal strDescription, ByVal strCreator)
    
    Dim objMTSCatalog, objPackage, lngPkgCount, intLoop, objNewPackage
    Dim lngNewID, objCompInPkg, objUtilInterface, objLocalComputer, objIIS_W3SVC
    On Error Resume Next
    Set objMTSCatalog = CreateObject("MTSAdmin.Catalog.1")
    If Err.Number <> 0 Then
        MsgBox "Unable to Communicate with MTS (Component Services) Catalog  - " & _
            "please ensure that MTS and DTC are running" & vbCrLf & _
            "    - Error Details: " & Err.Description & " [Number:" & _
            Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
        Err.Raise Err.Number
        Exit Function
    End If
    Set objPackage = objMTSCatalog.GetCollection("Packages")
    If Err.Number <> 0 Then
        MsgBox "Unable to open MTS (Component Services) Packages collection  - " & _
            "please ensure that MTS and DTC are running" & vbCrLf & _
            "    - Error Details: " & Err.Description & " [Number:" & _
            Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
        Err.Raise Err.Number
        Exit Function
    End If
    Set objIIS_W3SVC = GetObject("IIS://localhost/W3SVC")
    If Err.Number <> 0 Then
        MsgBox "Unable to open IIS W3SVC Root object - " & _
            "please ensure that IIS is running" & vbCrLf & _
            "    - Error Details: " & Err.Description & " [Number:" & _
            Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
        Err.Raise Err.Number
        Exit Function
    End If
    
    objPackage.Populate
    lngPkgCount = objPackage.Count
    ' Remove any existing MTS package with the specified name
    ' Loop in reverse order because the index changes after the removal of a package
    For intLoop = (lngPkgCount - 1) To 0 Step -1
        If objPackage.Item(intLoop).Value("Name") = strName Then objPackage.Remove (intLoop)
    Next
    objPackage.SaveChanges
    
    If Not CBool(blnAdd) Then Exit Function

    ' Add new MTS package, and set properties
    Set objNewPackage = objPackage.Add
    lngNewID = objNewPackage.Key
    objNewPackage.Value("Name") = strName
    objNewPackage.Value("Activation") = "Local"
    objNewPackage.Value("Description") = strDescription
    objNewPackage.Value("CreatedBy") = strCreator
    objNewPackage.Value("Identity") = objIIS_W3SVC.Get("AnonymousUserName")
    objNewPackage.Value("Password") = objIIS_W3SVC.Get("AnonymousUserPass")
    objPackage.SaveChanges
    If Err.Number <> 0 Then
        MsgBox "Unable to save MTS Package - " & _
            "please ensure that MTS and DTC are running" & vbCrLf & _
            "    - Error Details: " & Err.Description & " [Number:" & _
            Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
        Err.Raise Err.Number
        Exit Function
    End If
    Set objCompInPkg = objPackage.GetCollection("ComponentsInPackage", lngNewID)
    Set objUtilInterface = objCompInPkg.GetUtilInterface
    ' Install components in MTS package
    For intLoop = 0 To UBound(arrDLLs)
        objUtilInterface.InstallComponent strPath & arrDLLs(intLoop), "", ""
        If Err.Number <> 0 Then
            MsgBox "Unable to install component (" & intLoop & ":" & strPath & _
                arrDLLs(intLoop) & ") in MTS package - " & _
                "please ensure that MTS and DTC are running" & vbCrLf & _
                "    - Error Details: " & Err.Description & " [Number:" & _
                Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
            Err.Raise Err.Number
            Exit Function
        End If
    Next
    ' Set MTS transaction timeout
    Set objLocalComputer = objMTSCatalog.GetCollection("LocalComputer")
    objLocalComputer.Populate
    objLocalComputer.Item(0).Value("TransactionTimeout") = strTimeout
    objLocalComputer.SaveChanges
    If Err.Number <> 0 Then
        MsgBox "Unable to save MTS updates - " & _
            "please ensure that MTS and DTC are running" & vbCrLf & _
            "    - Error Details: " & Err.Description & " [Number:" & _
            Hex(Err.Number) & "]", vbCritical, g_MsgBoxTitle
        Err.Raise Err.Number
        Exit Function
    End If
End Function

'===========================================================================
